@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 1998-1999  Thomas Roessler <roessler@guug.de>
+ *  Copyright (C) 1998-2000  Thomas Roessler <roessler@guug.de>
+ *  Copyright (C) 2000       Roland Rosenfeld <roland@spinnaker.de>
  *
  *  This program is free software; you can redistribute
  *  it and/or modify it under the terms of the GNU
@@ -15,12 +16,12 @@
  *
  *  You should have received a copy of the GNU General
  *  Public License along with this program; if not, write
- *  to the Free Software Foundation, Inc., 675 Mass Ave,
- *  Cambridge, MA 02139, USA.
+ *  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *  Boston, MA  02111, USA.
  *
  */
 
-/* $Id: fetchaddr.c,v 1.11 1999/04/29 14:57:08 roland Exp $ */
+/* $Id: fetchaddr.c,v 1.20 2001/09/01 09:59:11 roland Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,15 +32,17 @@
 #include "rfc822.h"
 #include "rfc2047.h"
 
+#define MAXHDRS 21
+
 struct header
 {
-  const char *tag;
+  char *tag;
   char *value;
   size_t len;
   size_t taglen;
 };
 
-struct header hdr[] =
+struct header hdr[MAXHDRS] =
 {
   { "to:",		NULL, 0,  3 },
   { "from:",		NULL, 0,  5 },
@@ -55,7 +58,8 @@ void chop(struct header *cur)
     cur->value[--cur->len] = '\0';
 }
 
-int writeout(struct header *h, const char *datefmt)
+int writeout(struct header *h, const char *datefmt, 
+	     unsigned char create_real_name)
 {
   int rv = 0;
   ADDRESS *addr, *p;
@@ -72,6 +76,17 @@ int writeout(struct header *h, const char *datefmt)
   rfc2047_decode_adrlist(addr);
   for(p = addr; p; p = p->next)
   {
+    if(create_real_name == 1 
+       && (!p->personal || !*p->personal)
+       && p->mailbox)
+    {
+      if(p->personal)
+	FREE(p->personal);
+      p->personal = safe_strdup(p->mailbox);
+      c=strchr(p->personal, '@');
+      if (c)
+	*c='\0';
+    }
     if(!p->group && p->mailbox && *p->mailbox && p->personal)
     {
       if(p->personal && strlen(p->personal) > 30)
@@ -101,7 +116,55 @@ int main(int argc, char* argv[])
   int i, rv;
   int partial = 0;
   struct header *cur_hdr = NULL;
-  char* datefmt;
+  char *datefmt = NULL;
+  char *headerlist = NULL;
+  char *fieldname, *next;
+  char create_real_name = 0;
+
+  /* process command line arguments: */
+  if (argc > 1) {
+    i = 1;
+    while (i < argc) {
+      if (!strcmp (argv[i], "-d") && i+1 < argc) {
+	datefmt = argv[++i];
+      } else if (!strcmp (argv[i], "-x") && i+1 < argc) {
+	headerlist = argv[++i];
+      } else if (!strcmp (argv[i], "-a")) {
+	create_real_name = 1;
+      } else {
+	fprintf (stderr, "%s: `%s' wrong parameter\n", argv[0], argv[i]);
+      }
+      i++;
+    }
+  }
+
+  if (!datefmt) 
+    datefmt = safe_strdup("%Y-%m-%d %H:%M");
+
+  if (headerlist && strlen (headerlist) > 0 ) {
+    fieldname = headerlist;
+    i = 0;
+    while ( i < MAXHDRS-1 && (next = strchr (fieldname, ':'))) {
+      hdr[i].tag = safe_malloc (next - fieldname + 2);
+      strncpy (hdr[i].tag, fieldname, next - fieldname);
+      hdr[i].tag[next - fieldname] = ':';
+      hdr[i].tag[next - fieldname + 1] = '\0';
+      hdr[i].taglen = next - fieldname + 1;
+      fieldname = next+1;
+      i++;
+    }
+    
+    if (i < MAXHDRS-1 && *fieldname != '\0') {
+      hdr[i].tag = safe_malloc (strlen (fieldname) + 2);
+      strncpy (hdr[i].tag, fieldname, strlen (fieldname));
+      hdr[i].tag[strlen (fieldname)] = ':';
+      hdr[i].tag[strlen (fieldname) + 1] = '\0';
+      hdr[i].taglen = strlen (fieldname) + 1;
+      i++;
+    }
+    
+    hdr[i].tag = NULL;	/* end of hdr list */
+  }
 
   while(fgets(buff, sizeof(buff), stdin))
   {
@@ -146,16 +209,10 @@ int main(int argc, char* argv[])
       partial = 0;
   }
 
-  if (argc == 1) {
-    datefmt = strdup("%Y-%m-%d %H:%M");
-  } else {
-    datefmt = argv[1];
-  }
-
   for(rv = 0, i = 0; hdr[i].tag; i++)
   {
     if(hdr[i].value)
-      rv = writeout(&hdr[i], datefmt) || rv;
+      rv = writeout(&hdr[i], datefmt, create_real_name) || rv;
   }
 
   return (rv ? 0 : 1);
